package com.rshy.project.hy.manager.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.EnumUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.rshy.project.hy.manager.RshyJkManager;
import com.rshy.project.hy.model.constant.ItfwConstant;
import com.rshy.project.hy.model.dto.NucleicAcidDTO;
import com.rshy.project.hy.model.entity.Identity;
import com.rshy.project.hy.model.enums.*;
import com.rshy.project.hy.server.ItfwServer;
import com.rshy.project.hy.server.VIIDServer;
import com.rshy.project.hy.server.YituFaceServer;
import com.rshy.project.hy.server.dto.*;
import com.rshy.project.hy.server.param.RshyParam;
import com.rshy.project.hy.server.param.RshyQuickParam;
import com.rshy.project.hy.server.vo.RetrievaResultVO;
import com.rshy.project.hy.server.vo.RetrievalRepositoryVO;
import com.rshy.project.hy.server.vo.RshyVo;
import com.rshy.project.hy.service.IdentityService;
import com.rshy.project.hy.util.ObjectUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.DependsOn;
import org.springframework.core.env.Environment;
import org.springframework.core.task.support.ExecutorServiceAdapter;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * @author: lileilei
 * @date: 2022/4/23 20:12
 * @description:
 */
@Component
@Slf4j
@DependsOn("itfwServer")
@EnableAsync
public class RshyJkManagerImpl implements RshyJkManager {
    @Resource
    private YituFaceServer yituFaceServer;
    @Resource
    private ItfwServer itfwServer;
    @Resource
    private VIIDServer viidServer;
    @Resource
    private StringRedisTemplate stringRedisTemplate;
    @Resource
    private ThreadPoolTaskExecutor poolTaskExecutor;
    @Resource
    private IdentityService identityService;
    @Resource
    private Environment environment;

    @Override
    @Async("hstTaskExecutor")
    public Future<RshyVo> getRshyInfo(RshyParam rshyParam) {
        // timeout 转成毫秒 这里考虑到网络传输延时等原因，减去300ms
        rshyParam.setTimeout((int) TimeUnit.MILLISECONDS.convert(rshyParam.getTimeout(), TimeUnit.SECONDS));
        int jkmTimeout = rshyParam.getTimeout() - 500;
        RshyVo rshyVo = new RshyVo();
        PctgkryDTO gkryDTO = new PctgkryDTO();
        ResultDTO resultDTO = new ResultDTO();
        resultDTO.setPctgkryDTO(gkryDTO);
        DateTime beginDate = DateTime.now();
        PctjkmInfoDTO jkmInfo = null;

        // 刷健康码或场所码
        if (StringUtils.isNotBlank(rshyParam.getBarCode()) && StringUtils.isBlank(rshyParam.getSfzh())) {
            try {
                //查询健康码信息
                jkmInfo = itfwServer.getPctjkm(rshyParam.getBarCode(), StrUtil.EMPTY, jkmTimeout);
            } catch (Exception e) {
                log.error("查询健康码信息异常:{}", e.getMessage());
                //e.printStackTrace();
            }
        }
        if (jkmInfo == null && StrUtil.isBlank(rshyParam.getSfzh())) {
            return new AsyncResult<>(rshyVo.setPassStatus(PassStatusEnum.NO_HEALTH_CODE).setSfzh(rshyParam.getSfzh()).setXm(rshyParam.getXm()).setWarning(PassStatusEnum.NO_HEALTH_CODE.getDesc())
                    .setNucleic_acid(new NucleicAcidDTO().set(NucleicAcidEnum.NO_DATA)));
        }

        if (jkmInfo != null) {
            rshyParam.setSfzh(jkmInfo.getId_num());
            rshyVo.setSfzh(jkmInfo.getId_num()).setXm(jkmInfo.getUser_name()).setHealthCode(jkmInfo.getColor_code_label()).setPhone(jkmInfo.getPn());
            resultDTO.setPctjkmInfoDTO(jkmInfo);
            if (rshyParam.getIdentity() == null) {
                saveBarcode2DB(jkmInfo.getBar_code(), jkmInfo.getId_num());
            } else {
                rshyVo.setPhoto(rshyParam.getIdentity().getPhoto());
            }
        } else {
            rshyVo.setSfzh(rshyParam.getSfzh());
            if (rshyParam.getIdentity() == null) {
                saveBarcode2DB(StrUtil.EMPTY, rshyParam.getSfzh());
            } else {
                rshyVo.setXm(rshyParam.getIdentity().getName());
                rshyVo.setPhone(rshyParam.getIdentity().getPhone());
                rshyVo.setAddress(rshyParam.getIdentity().getAddress());
                rshyVo.setPhoto(rshyParam.getIdentity().getPhoto());
            }
        }
        int jkmUsedTime = (int) DateUtil.between(beginDate, DateTime.now(), DateUnit.MS);
        System.out.println("健康码接口用时: " + jkmUsedTime + "毫秒");

        rshyParam.setTimeout(rshyParam.getTimeout() - jkmUsedTime - 600);
        //剩余时间不到50ms，直接返回
        if (rshyParam.getTimeout() <= 50) {
            return new AsyncResult<>(rshyVo);
        }

        log.info(rshyParam.toString());

        AtomicReference<RshyVo> quickRshyVo = new AtomicReference<>(new RshyVo());
        List<Runnable> taskList = new ArrayList<Runnable>() {
            {
                if (resultDTO.getPctjkmInfoDTO() == null) {
                    add(() -> resultDTO.setPctjkmInfoDTO(itfwServer.getPctjkm("", rshyParam.getSfzh(), rshyParam.getTimeout())));
                }
                add(() -> resultDTO.setPctsfzResult(itfwServer.getPctsfz(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctymDataDTOS(itfwServer.getPctym(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPcthsResultDTO(itfwServer.getPcths(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctxcmResultDTO(itfwServer.getPctxcm(rshyParam.getSfzh(), rshyParam.getTimeout())));
                if (StrUtil.isBlank(rshyParam.getXm())) {   // 刷健康码或者手动输入身份证号码，获取证件照片
                    if (rshyVo.getPhoto() == null) {
                        add(() -> resultDTO.setPctzjzpResultDTO(itfwServer.getPctzjzp(rshyParam.getSfzh(), rshyParam.getTimeout())));
                    }
                } else {
                    add(() -> itfwServer.savePhoto2DB(rshyParam.getSfzh(), rshyParam.getPhoto()));
                    rshyVo.setPhoto(null);
                }
            }
        };

        if (rshyParam.getScreenLocked() == 0) {
            //没锁屏的情况下，继续进行七大类管控接口、人车核录接口、杭州市重点人员底库数据查询接口以及依图的比对
            if (StrUtil.isNotBlank(rshyParam.getPhoto())) {
                //add(() -> resultDTO.setPersonIdentifyDTO(viidServer.personIdentify(rshyParam.getPhoto(), rshyParam.getTimeout())));
                taskList.add(() -> quickRshyVo.set(getYitu(rshyParam.getPhoto(), (rshyParam.getTimeout()))));
            }
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctdfkDTO(itfwServer.getPctdfk(rshyParam.getSfzh(), "", rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctjsryDTO(itfwServer.getPctjsry(rshyParam.getSfzh(), rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctztDTO(itfwServer.getPctzt(rshyParam.getSfzh(), rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctzfbaDTO(itfwServer.getPctzfba(rshyParam.getSfzh(), rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctqkryDTO(itfwServer.getPctqkry(rshyParam.getSfzh(), rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctxdryDTO(itfwServer.getPctxdry(rshyParam.getSfzh(), "", rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctzdryDTO(itfwServer.getPctzdry(rshyParam.getSfzh(), rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.setPctrchlDTO(itfwServer.getPctrchl(rshyParam.getSfzh(), rshyParam.getLoginInfo(), rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.setPctzdrydkResultDTO(itfwServer.getPctzdrydk(rshyParam.getSfzh(), rshyParam.getTimeout())));
        }


        //异步流并发处理
//        taskList.parallelStream().forEach(v -> {
//            try {
//                v.run();
//            } catch (Exception e) {
//                log.error("异常信息：{}", e.getMessage());
//                e.printStackTrace();
//            }
//        });

        try {
            log.info("{}个任务, 用时 {}毫秒", taskList.size(), startTaskAllInOnce(taskList, rshyParam.getTimeout() + 100));
        } catch (InterruptedException e) {
            log.error("异常信息：{}", e.getMessage());
            e.printStackTrace();
        }

        log.info("IT服务平台接口调用完成");
        //用身份证号码获取到健康码信息
        if (resultDTO.getPctjkmInfoDTO() != null) {
            jkmInfo = resultDTO.getPctjkmInfoDTO();
            rshyVo.setXm(resultDTO.getPctjkmInfoDTO().getUser_name()).setHealthCode(resultDTO.getPctjkmInfoDTO().getColor_code_label()).setPhone(resultDTO.getPctjkmInfoDTO().getPn());
        }
        //今日进站次数
        rshyVo.setTimes(times(rshyVo.getSfzh()));
        //疫苗
        rshyVo.setVaccine(CollectionUtil.isEmpty(resultDTO.getPctymDataDTOS()) ? 0 : resultDTO.getPctymDataDTOS().size());

        //核酸报告
        rshyVo.setNucleic_acid(nucleicAcid(resultDTO.getPcthsResultDTO()));
        //行程卡
        rshyVo.setTravel_card(resultDTO.getPctxcmResultDTO() != null && resultDTO.getPctxcmResultDTO().getData() != null && resultDTO.getPctxcmResultDTO().getData().getResult().size() > 0 ? 1 : 0);
        //性别 从身份证号码中获取
        rshyVo.setXb(SexEnum.codeOf(IdcardUtil.getGenderByIdCard(rshyVo.getSfzh())).getDesc());
        //出生日期 从身份证号码中获取
        Date date = DateUtil.parse(IdcardUtil.getBirthByIdCard(rshyVo.getSfzh()));
        rshyVo.setBirthday(DateUtil.format(date, "yyyy-M-d"));
        //取地址、姓名、民族
        if (resultDTO.getPctsfzResult() != null) {
            //有警察叔叔身份证接口返回数据
            rshyVo.setXm(resultDTO.getPctsfzResult().getCZRKXM());
            rshyVo.setAddress(resultDTO.getPctsfzResult().getCZRKZZ());
            rshyVo.setMz(resultDTO.getPctsfzResult().getCZRKMZ());
        } else {
            //户籍省份 从身份证号码中获取
            rshyVo.setAddress(IdcardUtil.getProvinceByIdCard(rshyVo.getSfzh()));
        }
        if (StrUtil.isNotBlank(rshyParam.getXm()) && rshyVo.getXm() == null) {
            rshyVo.setXm(rshyParam.getXm());
        }
        //人车核录核查结果
        String hcjg = resultDTO.getPctrchlDTO() == null ? "" : resultDTO.getPctrchlDTO().getHcjg();

        boolean hsenable = Boolean.parseBoolean(environment.getProperty("zmt.hshy", "true"));

        if (hsenable && rshyVo.getNucleic_acid().getStatus() == 0) {
            rshyVo.setPassStatus(PassStatusEnum.NON_CONFORMANCE).setKind(PassStatusEnum.NON_CONFORMANCE.getDesc()).setWarning(PassStatusEnum.NON_CONFORMANCE.getDesc());
        } else if (hsenable && rshyVo.getTravel_card() != 0) {
            //行程码到过中高风险地区
            rshyVo.setPassStatus(PassStatusEnum.ABNORMAL_TRAVEL_CARD).setKind(PassStatusEnum.ABNORMAL_TRAVEL_CARD.getDesc());
        } else if (jkmInfo != null && PassStatusEnum.RED_CODE.getDesc().equalsIgnoreCase(jkmInfo.getColor_code_label())) {
            //红码
            rshyVo.setPass(PassStatusEnum.RED_CODE.getCode()).setTts(PassStatusEnum.RED_CODE.getTts()).setDisplayMsg(PassStatusEnum.RED_CODE.getDesc()).setWarning(PassStatusEnum.RED_CODE.getDesc()).setKind("健康码" + PassStatusEnum.RED_CODE.getDesc());
        } else if (jkmInfo != null && PassStatusEnum.YELLOW_CODE.getDesc().equalsIgnoreCase(jkmInfo.getColor_code_label())) {
            //黄码
            rshyVo.setPassStatus(PassStatusEnum.YELLOW_CODE).setKind("健康码" + PassStatusEnum.YELLOW_CODE.getDesc()).setWarning(PassStatusEnum.YELLOW_CODE.getDesc());
        } else if (StrUtil.containsAnyIgnoreCase(hcjg, RchljgEnum.CAPTURE.getDesc()) || StrUtil.containsAnyIgnoreCase(hcjg, RchljgEnum.INTERCEPT.getDesc())) {
            //人车核录接口返回抓捕或者拦截
            if (StrUtil.isNotBlank(resultDTO.getPctrchlDTO().getXm())) {
                rshyVo.setXm(resultDTO.getPctrchlDTO().getXm());
                rshyVo.setMz(EnumUtil.likeValueOf(Nation.class, resultDTO.getPctrchlDTO().getMz()).getDesc());
            }
            rshyVo.setPassStatus(PassStatusEnum.NO_PASSING).setKind(kind(resultDTO)).setWarning(warning(resultDTO));
        } else if (resultDTO.getPctgkryDTO().getPctztDTO() != null && resultDTO.getPctgkryDTO().getPctztDTO().getRyztmc().equalsIgnoreCase("在逃")) {
            //在逃接口返回并且ryztmc字段为在逃
            rshyVo.setXm(resultDTO.getPctgkryDTO().getPctztDTO().getXm());
            rshyVo.setPassStatus(PassStatusEnum.NO_PASSING).setKind(kind(resultDTO)).setWarning(warning(resultDTO));
        } else if (!ObjectUtils.allFieldIsNULL(resultDTO.getPctgkryDTO()) || StrUtil.containsAnyIgnoreCase(hcjg, RchljgEnum.IN_DOUBT.getDesc())) {
            //七大类管控人员和人车核录接口返回存疑人员
            rshyVo.setPassStatus(PassStatusEnum.RIGHT_WAY).setKind(kind(resultDTO)).setWarning(warning(resultDTO));
        } else if (resultDTO.getPctzdrydkResultDTO() != null && resultDTO.getPctzdrydkResultDTO().getTotal() > 0) {
            //杭州市重点人员底库查询接口
            rshyVo.setPassStatus(PassStatusEnum.RIGHT_WAY).setKind(resultDTO.getPctzdrydkResultDTO().getData().get(0).getRyflList()[0]);
        } else if (quickRshyVo.get() != null && StrUtil.isNotEmpty(quickRshyVo.get().getSfzh())) {
            //依图接口
            rshyVo.copySomeValues(quickRshyVo.get());
        } else {
            //正常通过
            rshyVo.setPassStatus(PassStatusEnum.NORMAL_TRAFFIC);
        }

        if (rshyVo.getPass().equals(PassStatusEnum.NORMAL_TRAFFIC.getCode()) && rshyVo.getHealthCode() != null) {
            rshyVo.setTts(rshyVo.getHealthCode());
        }
        if (resultDTO.getPctzjzpResultDTO() != null) {
            rshyVo.setPhoto(resultDTO.getPctzjzpResultDTO().getPhoto());
        }

        if (resultDTO.getPctjkmInfoDTO() != null) {
            saveIdentity2DB(resultDTO.getPctjkmInfoDTO().getBar_code(), rshyVo);
        } else {
            saveIdentity2DB(StrUtil.EMPTY, rshyVo);
        }
        return new AsyncResult<>(rshyVo);
    }


    /**
     * 并行执行任务
     *
     * @param task 任务列表
     * @return 用时，毫秒
     * @throws InterruptedException
     */
    protected long startTaskAllInOnce(List<Runnable> task, long timeout) throws InterruptedException {
        long startTime = System.currentTimeMillis();
        System.out.println(startTime + " [" + Thread.currentThread() + "] All thread is ready, concurrent going...");
        final CountDownLatch latch = new CountDownLatch(task.size());
        ExecutorService executorService = new ExecutorServiceAdapter(poolTaskExecutor);
        for (Runnable r : task) {
            executorService.submit(() -> {
                try {
                    r.run();
                } catch (Exception e) {
                    log.error("异常信息：{}", e.getMessage());
                } finally {
                    latch.countDown();
                }
            });
        }

        latch.await(timeout, TimeUnit.MILLISECONDS);
        long endTime = System.currentTimeMillis();
        System.out.println(endTime + " [" + Thread.currentThread() + "] All thread is completed.");
        return endTime - startTime;
    }


    /**
     * 保存身份信息到数据库
     *
     * @param barCode 健康码bar_code
     * @param rshyVo
     */
    @Async
    protected void saveIdentity2DB(String barCode, RshyVo rshyVo) {
        log.info("保存身份信息到数据库, 身份证号码: {}", rshyVo.getSfzh());
        Identity identity = new Identity();
        if (StrUtil.isNotBlank(barCode)) {
            identity.setBarCode(barCode);
        }
        identity.setIdNum(rshyVo.getSfzh());
        identity.setName(rshyVo.getXm());
        identity.setAddress(rshyVo.getAddress());
        identity.setPhone(rshyVo.getPhone());
        //identity.setPhoto(rshyVo.getPhoto());
        identityService.saveOK(identity);
    }

    @Async
    protected void saveBarcode2DB(String barCode, String idNum) {
        LambdaQueryWrapper<Identity> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Identity::getIdNum, idNum);
        Identity identity = identityService.getOne(wrapper);
        if (identity != null) return;
        identity = new Identity();
        if (StrUtil.isNotBlank(barCode)) {
            identity.setBarCode(barCode);
        }
        identity.setIdNum(idNum);
        identityService.save(identity);
    }

    /**
     * 组装返回的核酸信息
     *
     * @param pcthsResultDTO 第三方核酸接口返回数据
     * @return
     */
    private NucleicAcidDTO nucleicAcid(PcthsResultDTO pcthsResultDTO) {
        NucleicAcidDTO nucleicAcidDTO = new NucleicAcidDTO();
        if (pcthsResultDTO == null) {
            nucleicAcidDTO.setStatus(NucleicAcidEnum.NO_DATA.getCode()).setResults(NucleicAcidEnum.NO_DATA.getDesc());
            return nucleicAcidDTO;
        }

        if (pcthsResultDTO.getYU_CJSJ() != null) {
            DateTime parse = DateUtil.date(Long.parseLong(pcthsResultDTO.getYU_CJSJ()) * 1000);
            log.info("最近一次核酸采样时间: {}", parse.toString());
            long betweenMs = DateUtil.betweenMs(parse, DateUtil.date());
            //72h防疫规则
            if (betweenMs > 72 * 60 * 60 * 1000) {
                nucleicAcidDTO.set(NucleicAcidEnum.NOT_CONFORM);
            } else {
                nucleicAcidDTO.set(NucleicAcidEnum.CONFORM);
            }
            return nucleicAcidDTO;
        }

        nucleicAcidDTO.set(NucleicAcidEnum.NO_DATA_TIME);
        return nucleicAcidDTO;
    }

    @Override
    public RshyVo getQuickRyhyInfo(RshyQuickParam rshyQuickParam) {
        RshyVo rshyVo = getYitu(rshyQuickParam.getPhoto(), rshyQuickParam.getTimeout() * 1000);
        log.info("快速核验响应内容：{}", JSON.toJSONString(rshyVo));
        return rshyVo;
    }

    /**
     * 依图接口
     *
     * @param photo   人脸照片
     * @param timeout 超时时间 单位ms
     * @return
     */
    private RshyVo getYitu(String photo, int timeout) {
        RshyVo rshyVo = new RshyVo().setNucleic_acid(new NucleicAcidDTO().setStatus(NucleicAcidEnum.NO_DATA.getCode()).setResults(NucleicAcidEnum.NO_DATA.getDesc()));
        RetrievaResultVO retrievaResultVO = yituFaceServer.retrieval(photo, timeout);
        List<RetrievalRepositoryVO> voList = Convert.toList(RetrievalRepositoryVO.class, retrievaResultVO.getResults());
        if (CollectionUtil.isEmpty(voList)) {
            return rshyVo.setPass(PassStatusEnum.NORMAL_TRAFFIC.getCode()).setDisplayMsg(PassStatusEnum.NORMAL_TRAFFIC.getDesc()).setTts(PassStatusEnum.NORMAL_TRAFFIC.getTts());
        }

        //查询依图人像库信息
        RetrievaResultVO repository = yituFaceServer.repository(timeout);
        List<RepositoryDTO> dtoList = Convert.toList(RepositoryDTO.class, repository.getResults());
        Map<String, String> stringMap = CollectionUtil.isNotEmpty(dtoList) ? dtoList.stream().collect(Collectors.toMap(RepositoryDTO::getId, RepositoryDTO::getName, (v1, v2) -> v1)) : MapUtil.newHashMap();
        RetrievalRepositoryVO repositoryVO = voList.stream().findFirst().get();
        rshyVo.setPass(RepositoryEnum.COUNTRY_LARGE.getId().equals(repositoryVO.getRepository_id()) ? PassStatusEnum.NO_PASSING.getCode() : PassStatusEnum.RIGHT_WAY.getCode())
                .setKind(EnumUtil.likeValueOf(RepositoryEnum.class, repositoryVO.getRepository_id()).getName())
                .setWarning(stringMap.get(repositoryVO.getRepository_id())).setDisplayMsg(RepositoryEnum.COUNTRY_LARGE.getId().equals(repositoryVO.getRepository_id()) ? PassStatusEnum.NO_PASSING.getDesc() : PassStatusEnum.RIGHT_WAY.getDesc())
                .setTts(RepositoryEnum.COUNTRY_LARGE.getId().equals(repositoryVO.getRepository_id()) ? PassStatusEnum.NO_PASSING.getTts() : PassStatusEnum.RIGHT_WAY.getTts()).setHealthCode(StrUtil.EMPTY).setSfzh(repositoryVO.getPersonId())
                .setXm(repositoryVO.getName()).setXb(SexEnum.codeOf(repositoryVO.getGender()).getDesc()).setMz(StrUtil.EMPTY).setPhone(StrUtil.EMPTY).setAddress(StrUtil.EMPTY);//.setPhoto(rshyQuickParam.getPhoto());
        return rshyVo;
    }

    /**
     * 今日进站次数
     *
     * @param idNumber 身份证号
     * @return
     */
    private Integer times(String idNumber) {
        String cacheKey = idNumber + ":" + ItfwConstant.TIMES_KEY;
        int times = 1;
        String timesStr = stringRedisTemplate.opsForValue().get(cacheKey);
        if (timesStr == null) {
            //今日首次进站
            stringRedisTemplate.opsForValue().set(cacheKey, String.valueOf(times), DateUtil.endOfDay(DateUtil.date()).getTime() - DateUtil.date().getTime(), TimeUnit.MILLISECONDS);
        } else {
            times = Integer.parseInt(timesStr) + 1;
            stringRedisTemplate.opsForValue().increment(cacheKey);
        }

        return times;
    }

    /**
     * 人员类型简要概述
     *
     * @param resultDTO 所有第三方的管控人员接口返回数据
     * @return
     */
    private String kind(ResultDTO resultDTO) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctztDTO() != null ? resultDTO.getPctgkryDTO().getPctztDTO().getRyztmc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctdfkDTO() != null ? HyTypeEnum.pctdfk.getDesc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctjsryDTO() != null ? HyTypeEnum.pctjsry.getDesc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctxdryDTO() != null ? HyTypeEnum.pctxdry.getDesc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctqkryDTO() != null ? HyTypeEnum.pctqkry.getDesc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctzfbaDTO() != null ? HyTypeEnum.pctzfba.getDesc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctzdryDTO() != null ? HyTypeEnum.pctzdry.getDesc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctrchlDTO() != null ? resultDTO.getPctrchlDTO().getRylb() : StrUtil.EMPTY);
        return stringBuilder.toString();
    }

    /**
     * 警示信息
     *
     * @param resultDTO 所有第三方的管控人员接口返回数据
     * @return
     */
    private String warning(ResultDTO resultDTO) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctztDTO() != null ? resultDTO.getPctgkryDTO().getPctztDTO().getJyaq() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctdfkDTO() != null ? resultDTO.getPctgkryDTO().getPctdfkDTO().getJyaq() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctjsryDTO() != null ? resultDTO.getPctgkryDTO().getPctjsryDTO().getJyaq() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctxdryDTO() != null ? resultDTO.getPctgkryDTO().getPctxdryDTO().getXdcsmc() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctqkryDTO() != null ? resultDTO.getPctgkryDTO().getPctqkryDTO().getLsqkqk() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctzfbaDTO() != null ? resultDTO.getPctgkryDTO().getPctzfbaDTO().getAjmc() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctzdryDTO() != null ? resultDTO.getPctgkryDTO().getPctzdryDTO().getAjlbmc() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctrchlDTO() != null && !resultDTO.getPctrchlDTO().getHcjg().equals("通过") ? resultDTO.getPctrchlDTO().getBjxx() : StrUtil.EMPTY);
        return stringBuilder.toString();
    }
}
