package com.rshy.project.hy.service.impl;


import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.rshy.project.hy.mapper.IdentityMapper;
import com.rshy.project.hy.model.entity.Identity;
import com.rshy.project.hy.service.IdentityService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author fanqun
 * @since 2022-06-11
 */
@Service
@EnableCaching
@Slf4j
@EnableAsync
public class IdentityServiceImpl extends ServiceImpl<IdentityMapper, Identity> implements IdentityService {

    @Resource
    private IdentityMapper identityMapper;

    @Override
    @Cacheable(value = "identity", key = "#p0", unless = "#result==null || #result.name == null")
    public Identity getIdentity(String idNum) {
        LambdaQueryWrapper<Identity> wrapper = new LambdaQueryWrapper<>();
        if (StrUtil.isNotBlank(idNum)) {
            wrapper.eq(Identity::getIdNum, idNum);
        } else {
            return null;
        }
        return identityMapper.selectOne(wrapper);
    }

    @Override
    @Cacheable(value = "identity", key = "#p0", unless = "#result == null")
    public String getIdNum(String barCode) {
        return StrUtil.isBlank(barCode) ? null : identityMapper.getIdNum(barCode);
    }

    @Override
    @Async("hstTaskExecutor")
    @CachePut(value = "identity", key = "#p0.idNum", unless = "#result.name == null")
    public Identity saveOK(Identity identity) {
        if (super.saveOrUpdate(identity)) {
            LambdaQueryWrapper<Identity> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(Identity::getIdNum, identity.getIdNum());
            return identityMapper.selectOne(wrapper);
        }
        return null;
    }

    @Override
    @Async("hstTaskExecutor")
    public void savePhoto(String idNum, String photo) {
        log.info("保存证件照片到数据库, 身份证号码: {}", idNum);
        if (StrUtil.isBlank(idNum)) return;
        Map<String, String> map = identityMapper.getPhoto(idNum);
        if (map == null) {
            identityMapper.addPhoto(idNum, photo);
        } else if (StrUtil.isBlank(map.get("photo"))) {
            identityMapper.updatePhoto(idNum, photo);
        }

    }
}
